package com.taiji.system.oss;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.taiji.system.util.GlobalException;
import io.minio.*;
import io.minio.http.Method;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.PostConstruct;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/***
 * <p>
 * minio 模板
 * </p>
 * @author hou yi
 * @date 2022/11/21 11:01
 **/
public class MinIoTemplate {

    private static final Logger log = LoggerFactory.getLogger(MinIoTemplate.class);

    /*** MinIo 客户端*/
    private final MinioClient minioClient;

    /*** OSS 配置类*/
    private final OssProperties ossProperties;

    private final ChannelProperties channelProperties;

    public MinIoTemplate(MinioClient minioClient, OssProperties ossProperties, ChannelProperties channelProperties) {
        this.minioClient = minioClient;
        this.ossProperties = ossProperties;
        this.channelProperties = channelProperties;
    }

    /***
     *  存储桶是否存在
     * @param bucketName 存储桶名称
     * @return 、
     */
    public boolean bucketExists(String bucketName) {
        try {
            return minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
        } catch (Exception e) {
            e.printStackTrace();
            log.error("MinIo 判断存储桶是否存在失败...{}", e.getMessage());
            throw new GlobalException("MinIo 判断存储桶是否存在失败...");
        }
    }

    /**
     * 创建存储桶
     *
     * @param bucketName 桶名
     */
    public void makeBucket(String bucketName) {
        if (!bucketExists(bucketName)) {
            try {
                minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
            } catch (Exception e) {
                log.error("MinIo 创建存储桶失败...{}", e.getMessage());
                throw new GlobalException("MinIo 创建存储桶失败...");
            }
        }
    }

    /**
     * 上传文件
     *
     * @param file             流
     * @param originalFileName 原始文件名
     * @param bucketName       桶名
     * @param deduplication       true 同文件夹下生产唯一的文件名， false 同文件夹下相同文件名时直接覆盖
     * @return OssFile
     */
    public OssFile putObject(MultipartFile file, String bucketName, String originalFileName, boolean deduplication, String busCode) {
        String uuidFileName = deduplication ? generateOssUuidFileName(originalFileName, busCode) : generateOssFileName(originalFileName, busCode);
        try {
            if (!StringUtils.hasLength(bucketName)) {
                bucketName = ossProperties.getDefaultBucketName();
            }
            InputStream inputStream = file.getInputStream();
            minioClient.putObject(PutObjectArgs.builder()
                    .bucket(bucketName)
                    .object(uuidFileName)
                    .stream(inputStream, inputStream.available(), -1)
                    .contentType(MinIoContentType.getContentType(originalFileName.substring(originalFileName.lastIndexOf("."))))
                    .build());
            return new OssFile(uuidFileName, originalFileName);
        } catch (Exception e) {
            log.error("MinIo 上传文件失败...{}", e.getMessage());
            throw new GlobalException("MinIo 上传文件失败...请联系管理员");
        }
    }

    /**
     * 上传文件
     *
     * @param file             流
     * @param originalFileName 原始文件名
     * @param deduplication       true 同文件夹下生产唯一的文件名， false 同文件夹下相同文件名时直接覆盖
     * @return OssFile
     */
    public OssFile putObject(MultipartFile file, String appId, String key, String originalFileName, boolean deduplication) {
        List<ChannelPropertiesModel> jsonObject = JSONObject.parseArray(channelProperties.getChannelProperties(), ChannelPropertiesModel.class);
        ChannelPropertiesModel imodel = null;
        for (ChannelPropertiesModel model :
                jsonObject) {
            if (model.getAppId().equals(appId)) {
                imodel = model;
            }
        }
        if (imodel == null) {
            throw new GlobalException("当前appId未授权");
        }
        if (!imodel.getData().getAccessKey().equals(key)) {
            throw new GlobalException("key值校验失败，请输入正确的key");
        }
        ChannelModel data = imodel.getData();
        MinioClient minioClient = minioBuild(data);
        String uuidFileName = deduplication ? generateOssUuidFileName(originalFileName, "") : generateOssFileName(originalFileName,"");
        try {
            InputStream inputStream = file.getInputStream();
            minioClient.putObject(PutObjectArgs.builder()
                    .bucket("agent-issues")
                    .object(appId + "/" + uuidFileName)
                    .stream(inputStream, inputStream.available(), -1)
                    .contentType(MinIoContentType.getContentType(originalFileName.substring(originalFileName.lastIndexOf("."))))
                    .build());
            return new OssFile("agent-issues/" + appId + "/" + uuidFileName, originalFileName);
        } catch (Exception e) {
            log.error("MinIo 上传文件失败...{}", e.getMessage());
            throw new GlobalException("MinIo 上传文件失败...请联系管理员");
        }
    }

    private MinioClient minioBuild(ChannelModel data) {
        return MinioClient.builder()
                .endpoint(ossProperties.getEndpoint())
                .credentials(data.getAccessKey(), data.getSecretKey())
                .build();
    }


    /**
     * 返回临时带签名、过期时间一天、Get请求方式的访问URL
     *
     * @param bucketName  桶名
     * @param ossFilePath Oss文件路径
     * @return 、
     */
    public FileUrlVo getPresignedObjectUrl(String bucketName, String ossFilePath) {
        try {
            String imageUrl = minioClient.getPresignedObjectUrl(
                    GetPresignedObjectUrlArgs.builder()
                            .method(Method.GET)
                            .bucket(bucketName)
                            .object(ossFilePath)
                            .expiry(60 * 60 * 24)
                            .build());
            return new FileUrlVo(imageUrl);
        } catch (Exception e) {
            log.error("MinIo 访问文件URL临时路径失败...{}", e.getMessage());
            throw new GlobalException("MinIo 访问文件URL临时路径失败...");
        }
    }

    /**
     * 生成随机文件名，防止重复
     *
     * @param originalFilename 原始文件名
     * @return 、
     */
    public String generateOssUuidFileName(String originalFilename, String busCode) {
        // 原始文件名前加 24位uuid + "_"
        String uuid = UUID.randomUUID().toString().replace("-", "").substring(0, 24);
        if (!StringUtils.hasLength(busCode))
            return DateUtil.format(new Date(), "yyyyMMdd") + StrUtil.SLASH + uuid + "_" + originalFilename;
        else
            return DateUtil.format(new Date(), "yyyyMMdd") + StrUtil.SLASH + busCode + StrUtil.SLASH + uuid + "_" + originalFilename;
    }

    /**
     * 不处理文件名，重复
     *
     * @param originalFilename 原始文件名
     * @return 、
     */
    public String generateOssFileName(String originalFilename, String busCode) {
        // 原始文件名前加 24位uuid + "_"
        if (!StringUtils.hasLength(busCode))
            return DateUtil.format(new Date(), "yyyyMMdd") + StrUtil.SLASH + originalFilename;
        else
            return DateUtil.format(new Date(), "yyyyMMdd") + StrUtil.SLASH + busCode + StrUtil.SLASH + originalFilename;
    }

    /**
     * 初始化默认存储桶
     */
    @PostConstruct
    public void initDefaultBucket() {
        String defaultBucketName = ossProperties.getDefaultBucketName();
        if (bucketExists(defaultBucketName)) {
            log.info("MinIo 默认存储桶已存在...无需创建");
            return;
        }
        log.info("MinIo 创建默认存储桶...");
        makeBucket(ossProperties.getDefaultBucketName());
    }

    public MinioClient getMinioClient() {
        return minioClient;
    }

    public OssProperties getOssProperties() {
        return ossProperties;
    }

    public void getPresignedObjectUrl(String appId, String key, String ossFilePath, HttpServletResponse response) {
        List<ChannelPropertiesModel> jsonObject = JSONObject.parseArray(channelProperties.getChannelProperties(), ChannelPropertiesModel.class);
        ChannelPropertiesModel imodel = null;
        for (ChannelPropertiesModel model :
                jsonObject) {
            if (model.getAppId().equals(appId)) {
                imodel = model;
            }
        }
        if (imodel == null) {
            throw new GlobalException("当前appId未授权");
        }
        if (!imodel.getData().getAccessKey().equals(key)) {
            throw new GlobalException("key值校验失败，请输入正确的key");
        }
        ChannelModel data = imodel.getData();
        MinioClient minioClient = minioBuild(data);
        InputStream in = null;
        ServletOutputStream out = null;
        try {
            String imageUrl = minioClient.getPresignedObjectUrl(
                    GetPresignedObjectUrlArgs.builder()
                            .method(Method.GET)
                            .bucket("agent-issues")
                            .object(appId + "/" + ossFilePath)
                            .expiry(60 * 60 * 24)
                            .build());
            File file = getByFilePath(imageUrl);
            MultipartFile multipartFile = convert(file);
            // 获取文件的输入流
            in = new FileInputStream(file);

            // 推断文件的媒体类型
            MediaType mediaType = MediaType.APPLICATION_OCTET_STREAM;
            if (imageUrl.endsWith(".pdf")) {
                mediaType = MediaType.APPLICATION_PDF;
            } else if (imageUrl.endsWith(".jpg") || imageUrl.endsWith(".jpeg")) {
                mediaType = MediaType.IMAGE_JPEG;
            } else if (imageUrl.endsWith(".png")) {
                mediaType = MediaType.IMAGE_PNG;
            } else if (imageUrl.endsWith(".gif")) {
                mediaType = MediaType.IMAGE_GIF;
            }


            // 设置响应头
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + file.getName());
            response.setHeader(HttpHeaders.CONTENT_TYPE, mediaType.toString());
            out = response.getOutputStream();
            // 读取文件流
            int len = 0;
            byte[] buffer = new byte[1024 * 10];
            while ((len = in.read(buffer)) != -1) {
                out.write(buffer, 0, len);
            }
//            return new FileUrlVo(multipartFile);
        } catch (Exception e) {
            log.error("MinIo 访问文件URL临时路径失败...{}", e.getMessage());
            throw new GlobalException("MinIo 访问文件URL临时路径失败...");
        } finally {
            try {
                in.close();
                out.close();
            } catch (NullPointerException e) {
                log.error("responseFileStream stream close() error:NullPointerException" + e.toString());
            } catch (Exception e) {
                log.error("responseFileStream stream close() error:" + e.toString());
            }
        }
    }

    private File getByFilePath(String imageUrl) {
        String localFilePath = "/file.pdf";  // 替换为你想要保存的本地文件路径
        try {
            // 创建URL对象
            URL url = new URL(imageUrl);
            // 打开连接
            URLConnection connection = url.openConnection();
            // 获取输入流
            InputStream inputStream = connection.getInputStream();

            // 创建本地文件对象
            File localFile = new File(localFilePath);
            // 创建文件输出流
            FileOutputStream outputStream = new FileOutputStream(localFile);

            byte[] buffer = new byte[1024];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer))!= -1) {
                outputStream.write(buffer, 0, bytesRead);
            }

            // 关闭资源
            outputStream.close();
            inputStream.close();

            System.out.println("文件已成功保存到本地: " + localFilePath);
            return localFile;
        } catch (IOException e) {
            e.printStackTrace();
            throw new GlobalException("文件加载失败");
        }
    }

    public MultipartFile convert(File file) throws IOException {
        String fileName = file.getName();
        String contentType = "application/octet-stream";


        InputStream inputStream = new FileInputStream(file);


        // 使用 MockMultipartFile 构造函数创建 MultipartFile
        MultipartFile multipartFile = new MockMultipartFile(
                fileName,
                fileName,
                contentType,
                inputStream
        );


        return multipartFile;
    }
}
